@@ -1,25 +1,17 @@
------------------------------------------------------------------------------
--- |
--- Module      :  Data.Array.Parallel.Unlifted.Distributed
--- Copyright   :  (c) 2006 Roman Leshchinskiy
--- License     :  see libraries/ndp/LICENSE
--- 
--- Maintainer  :  Roman Leshchinskiy <rl@cse.unsw.edu.au>
--- Stability   :  experimental
--- Portability :  non-portable (GHC Extensions)
+-- | Distributed types and operations.
 --
--- Distributed types and operations.
+--   * This is an internal API and shouldn't need to be used directly.
+--     Client programs should use "Data.Array.Parallel.Unlifted"
 --
-
 module Data.Array.Parallel.Unlifted.Distributed (
   -- * Gang operations
-  Gang, forkGang, gangSize, sequentialGang, seqGang,
+  Gang, forkGang, gangSize,
 
   -- * Gang hacks
   theGang,
 
   -- * Distributed types and classes
-  DT, Dist,
+  DT(..),
 
   -- * Higher-order combinators
   mapD, zipWithD, foldD, scanD,
@@ -35,25 +27,24 @@ module Data.Array.Parallel.Unlifted.Distributed (
   zipD, unzipD, fstD, sndD,
 
   -- * Distributed arrays
-  lengthD, splitLenD, splitLengthD,
-  splitD, splitAsD, joinLengthD, joinD, splitJoinD,
-  splitSegdD, splitSD,
-  lengthUSegdD, lengthsUSegdD, indicesUSegdD, elementsUSegdD,
+  lengthD, splitLenD, splitLenIdxD,
+  splitD, splitAsD, joinLengthD, joinD, splitJoinD, joinDM,
+  glueSegdD, carryD,
   Distribution, balanced, unbalanced,
 
   -- * Permutations
   permuteD, bpermuteD, atomicUpdateD,
 
   -- * Debugging
-  fromD, toD
+  fromD, toD, debugD
 ) where
 
-import Data.Array.Parallel.Unlifted.Distributed.Gang (
-  Gang, forkGang, gangSize, sequentialGang, seqGang)
 import Data.Array.Parallel.Unlifted.Distributed.TheGang
-import Data.Array.Parallel.Unlifted.Distributed.Types
 import Data.Array.Parallel.Unlifted.Distributed.Combinators
 import Data.Array.Parallel.Unlifted.Distributed.Scalars
 import Data.Array.Parallel.Unlifted.Distributed.Arrays
+import Data.Array.Parallel.Unlifted.Distributed.USegd
 import Data.Array.Parallel.Unlifted.Distributed.Basics
+import Data.Array.Parallel.Unlifted.Distributed.Types
+import Data.Array.Parallel.Unlifted.Distributed.Gang (Gang, forkGang, gangSize)
 
