@@ -3,6 +3,8 @@ import Control.Monad
 import Data.Sequence (ViewL(..), empty, fromList, viewl, (<|), (|>), (><))
 import System.Random
 
+-- This test
+
 threads = 4
 samples = 5000
 
@@ -17,6 +19,7 @@ testRace t s = do
   iss <- threadRandoms t s
   return (isInterleavingOf (ref::[Int]) iss)
 
+threadRandoms :: Random a => Int -> Int -> IO [[a]]
 threadRandoms t s = do
   vs <- sequence $ replicate t $ do
     v <- newEmptyMVar
@@ -34,3 +37,4 @@ isInterleavingOf xs yss = iio xs (viewl $ fromList yss) EmptyL where
 
 fromViewL (EmptyL) = empty
 fromViewL (x :< xs) = x <| xs
+
